#include "animation.h"

using namespace std;

CAnimation::CAnimation(const bool& orientation_, const int& offset_, const int& nbr_images_, const char *seq_, const int& nbr_seq_, const string& file_, const SDL_Rect& coord_, const float& coef_resol_) : CGraphic(coord_, coef_resol_), up_down(false), slide_up(0), seq(seq_), nbr_seq(nbr_seq_), offset(offset_), cursor(0) { 

   // name file completed with sufix like name3.png
   string complet_file;

   // to convert int to string
   CConverter conv;

   // init slide image alpha
   slide_blk = {1.0f, 1.0f, 1.0f, 1.0f};

   try { 

      for(int i = offset; i < offset + nbr_images_; ++i) {

         // we create a complet file like name to name12.png
         complet_file = file_ + (conv << (i + 1)) + ".png";

         // we create list with images

         shared_ptr<CDraw> draw = make_shared<CDraw>(complet_file.c_str(), coord_, coef_resol);

         // only texture is saved, not surface because we don't use tests on surface
         draw->Free_surface();

         // We reverse picture on Y axis
         if(orientation_)
            draw->reverse_H_SDL();

         Vimages.push_back(draw);

      }
   }
   catch(string &caught) {

      Vimages.clear();	   
      cerr << caught << endl;
      throw;

   }

   // we update width and height animation values
   get_object_properties(Vimages.at(0)->object_properties());

}

CAnimation::~CAnimation() {

      Vimages.clear();	   

}

bool CAnimation::slide_black(float speed) {

   // init slide_blk to start
   if(slide_up == 0 && slide_blk.r == 1.0)
   slide_blk = {.0f, .0f, .0f, 1.0f};

   if(!slide_up) {
      slide_blk.r += speed;
      slide_blk.g += speed;
      slide_blk.b += speed;
   }
   if(slide_up == 1) {
      slide_blk.r -= speed;
      slide_blk.g -= speed;
      slide_blk.b -= speed;
   }

   if(slide_blk.r >= 1.0f) {
      slide_up = 1;
      return false;
   }

   if(slide_blk.r <= .0f) {
      slide_up = 2;
      return false;
   }

   return true;
}

void CAnimation::show(const int& type_anim, const bool& orientation, const int& step, const float& deep) {

   if(seq)

      // we show the surface with id = cursor
      Vimages.at(seq[cursor])->show_GL(orientation, object_properties(), deep, slide_blk);

   else

      // we show the surface with a normal seq (1, 2, 3, 4, 5, ...)
      Vimages.at(cursor)->show_GL(orientation, object_properties(), deep, slide_blk);

   // increment animation
   pass(type_anim, step);
}

void CAnimation::show(const int& loop_type, const bool& orientation, const int& step, const TVector2d<float>& scale, const float& deep) {

   if(seq)

      // we show the surface with id = cursor
      Vimages.at(seq[cursor])->show_GL(orientation, object_properties(), scale, deep);

   else

      // we show the surface with a normal seq (1, 2, 3, 4, 5, ...)
      Vimages.at(cursor)->show_GL(orientation, object_properties(), scale, deep);

   // increment animation
   pass(loop_type, step);
}
