#include "box.h"

using namespace std;

CBox::CBox(const string& file_background, const SDL_Rect& size_, const float& deep_, const float& coef_resol_) : size(size_), deep(deep_), CGraphic(origins, coef_resol_)
{

   SDL_Rect coord;
   coord.x = (SIZE_TEXTURE_W_MAX - size.x) / 2;
   coord.y = (SIZE_TEXTURE_H_MAX - size.y) / 2;
   new (this) CGraphic(coord, coef_resol_);

   // we init text parameters
   font = NULL;
   COLOR_BLACK(color_text)
   rect_text.x = X_TXT;
   rect_text.y = Y_TXT;

   try {

      font = TTF_OpenFont(FONTS);
      if(!font)
         throw string("box.cpp : Can't load fonts");

      // create the box surface	
      Odraw_box_surf = new CDraw(file_background, object_properties(), coef_resol);
//      box_surf = SDL_CreateRGBSurface(SDL_HWSURFACE, size_.x, size_.y, 32, 0, 0, 0, 0);
 
      if(!Odraw_box_surf)
         throw string("box.cpp : Can't create back surface");

      // we update object rect with width and hight
      get_object_properties(Odraw_box_surf->get_properties());

      // paint all box surface with a single color
      //const SDL_Color color_back = COLOR_BACK;
      //Odraw_box_surf = new CDraw(box_surf, object_properties(), coef_resol);
      //Odraw_box_surf->full_color_SDL(color_back);

   }
   catch(string caught) {
		
      if(font)  
         TTF_CloseFont(font);

      delete(Odraw_box_surf);

      cerr << caught << endl;
      throw;
   }

   // We create buttons yes, no and ok      

   try {

      rect_button_yes.x = W_MARGE + object_properties().x;
      rect_button_no.x = object_properties().x + size.x - W_MARGE - W_BUTTON;
      rect_button_ok.x = object_properties().x + size.x / 2 - W_BUTTON / 2;
      rect_button_yes.y = rect_button_no.y = rect_button_ok.y = size.y - H_MARGE + object_properties().y;

      button_yes = new CButton(rect_button_yes, YES, deep + .1f, coef_resol);
      button_no = new CButton(rect_button_no, NO, deep + .1f, coef_resol);
      button_ok = new CButton(rect_button_ok, OK, deep + .1f, coef_resol);

   }
   catch(string caught) {

      delete(button_yes);
      delete(button_no);
      delete(button_ok);

      cerr << caught << endl;
      throw;
   }
}

CBox::~CBox() {

   delete button_yes;
   delete button_no;
   delete button_ok;
 
   if(font)  
      TTF_CloseFont(font);
		
   delete(Odraw_box_surf);

}

Sevents_key_joy CBox::show(vector<string>& Smessg, SDL_Event& event, const int& ok_yes_no) {

   // to change lines down
   SDL_Rect rect_text_move = rect_text;

   SDL_Surface *text_surface;

   // 0 to number lines of message
   for(vector<string>::iterator i = Smessg.begin(); i != Smessg.end(); ++i) {

      // we put the text on the box surface
      try {
         text_surface = TTF_RenderText_Solid(font, i->c_str(), color_text);
         Odraw_box_surf->blit_SDL(true, text_surface, NULL, rect_text_move);
         rect_text_move.y += H_LETTER + 2;

      }
      catch(string caught) {

         delete(button_yes);
         delete(button_no);
         delete(button_ok);

         //if(box_surf) {
         //   SDL_FreeSurface(box_surf);
         //   box_surf = NULL;
         //}

         if(text_surface) {
            SDL_FreeSurface(text_surface);
            text_surface = NULL;
         }

         delete(Odraw_box_surf);

         if(font)
            TTF_CloseFont(font);

         cerr << caught << endl;
         throw;

      }
   }

   // we delete text_surface
   SDL_FreeSurface(text_surface);
   text_surface = NULL;

   // we erase events buffered
   SDL_PumpEvents();

   // we draw the box
   glDisable(GL_BLEND);
   Odraw_box_surf->show_text_GL(false, object_properties(), TVector2d<float>((float)size.x / (float)Odraw_box_surf->surface_properties().w, (float)size.y / (float)Odraw_box_surf->object_properties().h), deep); 
   glEnable(GL_BLEND);

   // we show buttons
   if(ok_yes_no == YES_NO) {
       button_yes->show_off();
       button_no->show_off();
   }

   // we refresh and show the buffer
   glFlush();
   SDL_GL_SwapBuffers();

   // we test evaluation mouse events on buttons and keys
   bool quit = false;

   int buff_hatmotion = SDL_HAT_CENTERED;

   while(!quit) {

      SDL_Delay(TimeLeft());

      while(SDL_PollEvent(&event)) {

         switch(event.type) {

            case SDL_QUIT :
               return {0, 0, 0};           

            case SDL_JOYBUTTONDOWN :

               if(ok_yes_no == NO_BUTTONS)
                  return {event.type, event.jbutton.button, 0};           
            break;

            case SDL_JOYHATMOTION :

               if(ok_yes_no == NO_BUTTONS) {
                  if(buff_hatmotion != SDL_HAT_CENTERED && event.jhat.value == SDL_HAT_CENTERED)
                     return {event.type, buff_hatmotion, 0};

                  if(buff_hatmotion == SDL_HAT_CENTERED && event.jhat.value != SDL_HAT_CENTERED)
                     buff_hatmotion = event.jhat.value;
               }
            break;

            case SDL_KEYDOWN :

               if(event.key.keysym.sym == SDLK_q)
                  return {0, 0, 0};           

               else if(ok_yes_no == NO_BUTTONS)
                  return {event.type, event.key.keysym.sym, 0};

            break;
            case SDL_MOUSEMOTION :
            case SDL_MOUSEBUTTONDOWN :

               // we eval yes and no buttons
               if(ok_yes_no == YES_NO) {

                  if(button_yes->eval(event))
                     return {0, 0, YES};

                  if(button_no->eval(event))
                     return {0, 0, NO};
	       }

               // we eval ok button
               if(ok_yes_no == YES_) {
                  if(button_ok->eval(event))
                     return {0, 0, OK};
	       }

                  glFlush();
                  SDL_GL_SwapBuffers();
	 }
      }
   }
   return {0, 0, 0};
}
